bl_info = {
    "name": "Dental3DPlugin Undercut Remove",
    "description": "Creates a new object without undercuts.",
    "author": "Alexandre Martinet",
    "version": (0, 9, 0),
    "blender": (2, 65, 0),
    "warning": "", # used for warning icon and text in addons panel
    "category": "Object"
    }
import bpy
import bmesh
from mathutils import Vector
class undercut_remove(bpy.types.Operator):
    """Dental3DPlugin Undercut Removal"""                  
    bl_idname = "object.undercut_remove"                   
    bl_label = "Remove Undercuts (Dental3DPlugin)"         
    bl_options = {'REGISTER', 'UNDO'}                      
    def execute(self, context): #Called by blender when running the operator.
        def bounds(obj, local=False):
            local_coords = obj.bound_box[:]
            om = obj.matrix_world
            if not local:    
                worldify = lambda p: om * Vector(p[:]) 
                coords = [worldify(p).to_tuple() for p in local_coords]
            else:
                coords = [p[:] for p in local_coords]
            rotated = zip(*coords[::-1])
            push_axis = []
            for (axis, _list) in zip('xyz', rotated):
                info = lambda: None
                info.max = max(_list)
                info.min = min(_list)
                info.distance = info.max - info.min
                push_axis.append(info)
            import collections
            originals = dict(zip(['x', 'y', 'z'], push_axis))
            o_details = collections.namedtuple('object_details', 'x y z')
            return o_details(**originals)
        def add_plane(name, obj, offset):
            myvertex = []
            myfaces = []
            od = bounds(obj, False)
            mypoint = [(od.x.min - offset, od.y.min - offset, 0)]
            myvertex.extend(mypoint)
            mypoint = [(od.x.min - offset, od.y.max + offset, 0)]
            myvertex.extend(mypoint)
            mypoint = [(od.x.max + offset, od.y.max + offset, 0)]
            myvertex.extend(mypoint)
            mypoint = [(od.x.max + offset, od.y.min - offset, 0)]
            myvertex.extend(mypoint)
            myface = [(0, 1, 2, 3)]
            myfaces.extend(myface)
            mymesh = bpy.data.meshes.new(name)
            myobject = bpy.data.objects.new(name, mymesh)  
            mymesh.from_pydata(myvertex, [], myfaces)
            mymesh.update(calc_edges=True)
            return myobject
        bpy.ops.object.mode_set(mode = 'OBJECT') #Erreurs si mode incorrect
        scene = context.scene
        obj = scene.objects.active
        if obj.name == "Undercuts":
            return {'CANCELLED'} 
        if len(context.scene.objects) == 0:
            return {'CANCELLED'} 
        count = 0
        for o in context.scene.objects:
            if o.type == 'MESH':
                count +=1
        if count > 1:
            return {'CANCELLED'}
        max_dim = 120
        if obj.dimensions.x > max_dim or obj.dimensions.y > max_dim or obj.dimensions.z > max_dim:
            return {'CANCELLED'} 
        od = bounds(obj, False)
        plane_object = add_plane("Plane0",obj, 2.0)
        plane_object.location.x = 0
        plane_object.location.y = 0
        plane_object.location.z = od.z.min
        scene.objects.link(plane_object)
        plane_object_1 = add_plane("Undercuts",obj, 1.0)
        plane_object_1.location.x = 0
        plane_object_1.location.y = 0
        plane_object_1.location.z = od.z.max + 1.0
        scene.objects.link(plane_object_1)
        plane_object_1.select = False
        plane_object.select = True
        obj.select = True
        context.scene.objects.active = obj
        bpy.ops.object.join()
        context.scene.objects.active = plane_object_1
        bpy.ops.object.mode_set(mode = 'EDIT')
        for i in range(1,10):
            bpy.ops.mesh.subdivide()
        bpy.ops.object.mode_set(mode = 'OBJECT') 
        bpy.ops.object.modifier_add(type='SHRINKWRAP')
        shrinkwrap = bpy.data.objects["Undercuts"].modifiers["Shrinkwrap"]
        shrinkwrap.target = obj
        shrinkwrap.wrap_method = "PROJECT"
        shrinkwrap.use_positive_direction = False
        shrinkwrap.use_negative_direction = True
        shrinkwrap.use_project_z = True
        bpy.ops.object.modifier_apply(modifier=shrinkwrap.name)
        bpy.ops.object.modifier_add(type='REMESH')
        remesh = bpy.data.objects["Undercuts"].modifiers["Remesh"]
        remesh.mode = "SMOOTH"
        remesh.octree_depth = 9
        bpy.ops.object.modifier_apply(modifier=remesh.name)
        bpy.ops.object.modifier_add(type='SOLIDIFY')
        solidify = bpy.data.objects["Undercuts"].modifiers["Solidify"]
        solidify.use_rim = True
        solidify.use_rim_only = True
        solidify.thickness = 0.05 #Setting!
        solidify.offset = 0
        bpy.ops.object.modifier_apply(modifier=solidify.name)
        ob = bpy.context.active_object
        mat = bpy.data.materials.get("undercut_material")
        if mat is None:
            mat = bpy.data.materials.new(name="undercut_material")
        mat.diffuse_color = [0.3,0.8,0.3] #Green
        if ob.data.materials:
            #assign to 1st material slot
            ob.data.materials[0] = mat
        else:
            #no slots
            ob.data.materials.append(mat)
        bpy.ops.object.select_all(action='DESELECT')
        obj.select = True
        bpy.ops.object.delete() 
        return {'FINISHED'}
def menu_func(self, context):
    self.layout.operator(undercut_remove.bl_idname)
def register():
    bpy.utils.register_class(undercut_remove)
    bpy.types.VIEW3D_MT_object.append(menu_func) 
def unregister():
    bpy.utils.unregister_class(undercut_remove)
    bpy.types.VIEW3D_MT_object.remove(menu_func)